@@ -1,15 +1,17 @@
 /*
-@version: 0.2
-@author: Sudheer K. <scifi.guy@hotmail.com>
+@version: 0.3
+@author: Sudheer K. <scifi1947 at gmail.com>
 @license: GNU General Public License
 */
 
 #include "callrouter.h"
+#include "vicardbusadaptor.h"
 #include <dbusutility.h>
 #include <gconfutility.h>
 #include <QDebug>
 #include <QRegExp>
 #include <QDBusConnection>
+#include <QDBusMessage>
 #include <QTimer>
 #include <QStringListIterator>
 
@@ -20,12 +22,25 @@ static QString strLastDialedNumber = QString();
 CallRouter::CallRouter(QObject *parent) :
     QObject(parent)
 {
+        dbusAdaptor = new VicarDbusAdaptor(this);
         gconfUtility = new GConfUtility();
+        this->registerDBusService();
+        qDebug() << "Registered DBus Service " << APPLICATION_DBUS_SERVICE;
+
+        //If International call routing is already enabled, start monitoring calls
+        //Otherwise wait for the signal from the configuration app
+        if (gconfUtility->getGconfValueBoolean((char *)"routing_enabled")){
+            qDebug() << "Call routing enabled. Starting call monitor..";
+            this->startOutgoingCallMonitor();
+        }
 }
 
 CallRouter::~CallRouter(){
         delete gconfUtility;
         gconfUtility = 0;
+
+        delete dbusAdaptor;
+        dbusAdaptor = 0;
 }
 
 void CallRouter::registerDBusService(){
@@ -36,8 +51,7 @@ void CallRouter::registerDBusService(){
         exit(1);
     }
 
-    if (!connection.registerObject(APPLICATION_DBUS_PATH, this,
-            QDBusConnection::ExportScriptableSlots)) {
+    if (!connection.registerObject(APPLICATION_DBUS_PATH, this, QDBusConnection::ExportAdaptors)) {
         qDebug() << dbusUtility.getErrorMessage();
         exit(2);
     }
@@ -130,7 +144,7 @@ void CallRouter::disconnectFromDBusSignals(){
 
 }
 
-void CallRouter::startOutgoingCallMonitor(){
+bool CallRouter::startOutgoingCallMonitor(){
 
     // Connect to DBus to monitor all outgoing calls
 
@@ -152,11 +166,10 @@ void CallRouter::startOutgoingCallMonitor(){
         qDebug() << "Failed to connect to Dbus signal CreateRequested in interface " << CSD_CALL_INTERFACE;
         qDebug() <<"DBus Error: "<< dbusUtility.getErrorMessage();
     }
-
-
+    return success;
 }
 
-void CallRouter::stopOutgoingCallMonitor(){
+bool CallRouter::stopOutgoingCallMonitor(){
 
     this->stopCallStatusMonitors();
 
@@ -177,7 +190,7 @@ void CallRouter::stopOutgoingCallMonitor(){
         qDebug() << "Failed to disconnect from Dbus signal CreateRequested in interface "<< CSD_CALL_INTERFACE;
         qDebug() <<"DBus Error: "<< dbusUtility.getErrorMessage();
     }
-
+    return status;
 }
 
 void CallRouter::processOutgoingCall(const QDBusMessage& dbusMessage){
@@ -232,34 +245,44 @@ void CallRouter::processOutgoingCall(const QDBusMessage& dbusMessage){
      }
 }
 
-void CallRouter::callViaCallingCard(){
+QString CallRouter::callViaCallingCard(){
         //Now call the calling card number. This is generally a local and/or tollfree number
 
         QString strCallingCardNumber = gconfUtility->getGconfValueString("calling_card_number");
 
-        qDebug() << "Wait time elapsed. Initiating call to "<< strCallingCardNumber;
+        qDebug() << "Initiating call to "<< strCallingCardNumber;
 
-        QList<QVariant> argsToSend;
-        argsToSend.append(strCallingCardNumber);
-        argsToSend.append(0);
-
-        bool status = dbusUtility.sendMethodCall(CSD_SERVICE,
-                                                 CSD_CALL_PATH,
-                                             CSD_CALL_INTERFACE,
-                                             QString("CreateWith"),argsToSend);
+        bool status = this->placeCall(strCallingCardNumber);
 
         QString strUserMessage;
+        QString strErrorMessage;
         if (status){
             qDebug() << "Call initiated successfully. Connecting DBus slot for audio connection monitor";
              startCallStatusMonitors();
         }
         else {
             strUserMessage = QString("Unable to initiate new call to ").append(strCallingCardNumber);
-            qDebug() << dbusUtility.getErrorMessage();
+            strErrorMessage = dbusUtility.getErrorMessage();
+            qDebug() << strErrorMessage;
             strLastDialedNumber.clear();
         }
 
         dbusUtility.displayNotification(strUserMessage);
+        return strErrorMessage;
+}
+
+
+bool CallRouter::placeCall(QString number){
+
+    QList<QVariant> argsToSend;
+    argsToSend.append(number);
+    argsToSend.append(0);
+
+    bool status = dbusUtility.sendMethodCall(CSD_SERVICE,
+                                             CSD_CALL_PATH,
+                                         CSD_CALL_INTERFACE,
+                                         QString("CreateWith"),argsToSend);
+    return status;
 
 }
 
@@ -435,15 +458,24 @@ QString CallRouter::convertToDTMFCode(QString strNumber){
 
     if (!strNumber.isEmpty()){
 
-        //Get the format required by calling card from coniguration
-        QString qstrDTMFFormat = gconfUtility->getGconfValueString("dtmf_format");
-        int intDTMFDelay = gconfUtility->getGconfValueInteger("dtmf_delay");
+        //int intDTMFDelay = gconfUtility->getGconfValueInteger("dtmf_delay");
 
-        if (intDTMFDelay <1 ) intDTMFDelay = 1;
-        if (qstrDTMFFormat.isEmpty()) qstrDTMFFormat = "<Country Code><Area Code><Phone Number>";
+        //if (intDTMFDelay <1 ) intDTMFDelay = 1;
+
+        int intDTMFDelay = 1;
 
         //Add the prefix p so that there is some delay after the call is picked up by the automated system to send DTMF tones.
-        strDTMFCode = QString("").fill('p',intDTMFDelay);
+        strDTMFCode = QString("").fill('p',intDTMFDelay);        
+
+        //Now check whether we need a prefix
+        QString strDTMFPrefix = gconfUtility->getGconfValueString("dtmf_prefix");
+        if (!strDTMFPrefix.isEmpty()){
+            strDTMFCode = strDTMFCode.append(strDTMFPrefix);
+        }
+
+        //Get the format required by calling card from coniguration
+        QString qstrDTMFFormat = gconfUtility->getGconfValueString("dtmf_format");
+        if (qstrDTMFFormat.isEmpty()) qstrDTMFFormat = "<Country Code><Area Code><Phone Number>";
 
         /* Replace 00 (international dialing code) at the beginning
            and also replace any character other than the numbers 0-9 and p.
@@ -463,7 +495,6 @@ QString CallRouter::convertToDTMFCode(QString strNumber){
         else if (qstrDTMFFormat.startsWith("011")){
             strDTMFCode = strDTMFCode.append("011");
         }
-        //Default case - we don't need any prefix
 
         strDTMFCode = strDTMFCode.append(strNumber);
 
@@ -498,3 +529,63 @@ bool CallRouter::isExcludedNumber(QString strInternationalNumber){
     }
     return isExcluded;
 }
+
+//DBus Method used by external applications to check whether VICaR is enabled and running
+bool CallRouter::isRunning(){
+
+    //Verify Whether Call Routing is Enabled
+    bool isRoutingEnabled = gconfUtility->getGconfValueBoolean("routing_enabled");
+    if (isRoutingEnabled){
+        return true;
+    }
+    else{
+        return false;
+    }
+}
+
+//DBus Method used by external applications to call via VICaR
+QString CallRouter::callInternationalNumber(const QString& strInternationalNumber){
+
+    //QDBusConnection connection = dbusUtility.getConnection();
+    QString strReturnMessage = "";
+
+    qDebug() << "New call requested by external application. Destination number is " << strInternationalNumber;
+
+     if (strInternationalNumber.startsWith("+") ||
+         strInternationalNumber.startsWith("00"))
+     {
+         qDebug() << "International number "<< strInternationalNumber << " recognized. Starting proceedings..";
+
+         //Check whether this is one of the excluded country codes
+         if (!isExcludedNumber(strInternationalNumber)){
+             strLastDialedNumber = strInternationalNumber;
+             QString strErrorMessage = this->callViaCallingCard();
+             if (!strErrorMessage.isEmpty()){
+                 strReturnMessage = strErrorMessage;
+             }
+         }
+         else{
+             //Country code is excluded, place a call directly
+             qDebug() << "Country code is excluded, placing call directly";
+             bool status = this->placeCall(strInternationalNumber);
+             if (!status){
+                 strReturnMessage = dbusUtility.getErrorMessage();
+             }
+         }
+    }
+    else{
+        strReturnMessage = "VICaR Error: Phone number is not in international format. Use +123456789 format.";
+        //QDBusMessage dbusError = QDBusMessage();
+        //dbusError = dbusError.createErrorReply("org.maemo.vicar.InvalidNumberFormat",strErrorMessage);
+        //connection.send(dbusError);
+    }
+
+    qDebug() << strReturnMessage;
+
+    if (strReturnMessage.isEmpty()){
+        return QString("Success");
+    }
+    else{
+        return strReturnMessage;
+    }
+ }
